<?php 
/*
 * @package Quick2Cart FJ Extra Fields System plugin
* @copyright Copyright (C) 2014 FasterJoomla. All rights reserved.
* @license GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
* @link     http://www.fasterjoomla.com
*/

defined( '_JEXEC' ) or die( 'Restricted Access' );

class fjExtraFieldsHelper
{	

	static private $colorboxAdded = false;
	static private $colorboxLoaded = false;


	/*
	 * For direct inclusion by the module quick2cart (the cart module)
	 */
	
	/**
	 * Add an extra field for vat-included prices to each item in the cart.
	 * product_tax_percentage = 14.50
	 * product_value_incl_tax = 229.00
	 * @param unknown $cart
	 */
	static public function addVat(&$cart) {
		
		// let's find the store ids:
		foreach($cart as $cartItem) {
			if (!in_array($cartItem['store_id'],$store_ids))
				$store_ids[] = $cartItem['store_id'];
		}
		// now let's get the vat percentage for each store:
			
		foreach($store_ids as $store_id) {
			$vats[$store_id] = self::getVat($store_id);
		}
		
		//error_log(var_export($vats,true));
		
		// now apply the tax-added price:
		foreach($cart as $key => $cartItem) {
			$store_id = $cartItem['store_id'];
			$vat = (float)$vats[$store_id];
			$cart[$key]['product_tax_percentage'] = $vat;
			$cart[$key]['product_value_incl_tax'] = $cartItem['tamt']*(1+$vat/100);
		}
	}
	
	static public function getVat($store_id) {
		$fjExtraPlugin = JPluginHelper::getPlugin('system','qtc_fj_extra_fields');
		
		if ($fjExtraPlugin) {
			$params = new JRegistry($fjExtraPlugin->params);
		} else {
			$params = new JRegistry();
		}
		return self::getExtra($store_id,$params)->tax;
	}	
	/*
	 * For use by the plugin directly
	 */
	
	
	static public function getExtra($storeId,$params) {
		//error_log('getExtra '.$storeId);
		if ($storeId>0) {
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			$query->select('extra')->
			from('#__kart_store')->
			where('id='.$db->quote($storeId));
			$db->setQuery($query);
			if ($result = $db->loadResult()) {
				if (strlen($result)>2)
				{
					$extra = json_decode($result);
				}
			} else {
				JError::raiseWarning(500,"Cannot retrieve extra fields from db ". $db->getErrorMsg());
			}
		}
		
		if (!isset($extra))
		{
			$extra = new stdClass();
		}
		
		// now grab defaults from the base plugins:
		if (!isset($extra->tax)) {
			$taxPlugin = JPluginHelper::getPlugin('qtctax','qtc_tax_default');
			if ($taxPlugin) {
				$tparams = new JRegistry($taxPlugin->params);
				$extra->tax =$tparams->get('tax_per');
			} else {
				// defaults to use during the store creation
				$extra->tax = 0;
			}
		}
		if (!isset($extra->shipping) || !isset($extra->shippingLimit)) {
			$shippingPlugin = JPluginHelper::getPlugin('qtcshipping','qtc_shipping_default');
			if ($shippingPlugin) {
				$sparams = new JRegistry($shippingPlugin->params);
				if (!isset($extra->shipping))
					$extra->shipping = $sparams->get('shipping_per');
				if (!isset($extra->shippingLimit))
					$extra->shippingLimit = $sparams->get('shipping_limit');
			} else {
				// defaults to use during the store creation
				$extra->shipping = 20;
				$extra->shippingLimit = 100;
			}
		}
		// Load default terms and conditions & privacy if an article is specified
		// in the params configuration
		
		if (empty($extra->terms)) {
			$extra->terms = self::loadContentArticle($params->get('defaultTerms'));
		}
		if (empty($extra->privacy)) {
			$extra->privacy = self::loadContentArticle($params->get('defaultPrivacy'));
		}
		if (empty($extra->banner)) {
			error_log('Load default banner');
			$extra->banner = $params->get('defaultBanner');
		}
		return $extra;		
	}

	
	/**
	 * Load an article directly from the db and return its content
	 * used to return the default article for terms and conditions and privacy.
	 * @param unknown $articleId
	 */
	static public  function loadContentArticle($articleId) {
		$db = JFactory::getDbo();
		$sql = $db->getQuery(true);
		$sql->select(array('title','introtext',$db->quoteName('fulltext'),'id'))
		->from('#__content')
		->where('id = '.$db->quote($articleId));
		$db->setQuery($sql);
		$html = array();
		if ($row = $db->loadObject()) {
			$html[] = "<h2>".$row->title."</h2>";
			$html[] = "<div class='custom article'>".$row->introtext;
			if (!empty($row->fulltext)) {
				// let's add a readmore link:
				$html[] = "<br>".$row->fulltext;
			}
			$html[] = "</div>";
		} else {
				
		}
		return join("",$html);
	}
	
	/**
	 * Return the current cart's store ids,
	 * additionally warning if items in the cart belong to more than one store.
	 *
	 * Multiple store_ids may imply multiple shipping charges; it is the responsibility
	 * of the caller to use one or more as appropriate.
	 *
	 * An additional param $raiseWarningOnMultipleStores may be set to raise a
	 * suspiciously vague error message.
	 *
	 * @return array
	 */
	static public function getStoreId($raiseWarningOnMultipleStores = false) {
		$cart = self::getCart();
		$store_ids = array();
		foreach($cart as $cartItem) {
			if (!in_array($cartItem['store_id'],$store_ids))
				$store_ids[] = $cartItem['store_id'];
		}
		if (count($store_ids)>1 && $raiseWarningOnMultipleStores) {
			JError::raiseWarning(517, 'Items in the cart belong to more than one store! Extra params may be misleading');
		}
	
		return $store_ids;
	}
	
	/**
	 * Invoke the cart model to return the current cart.
	 * @return cartItems
	 */
	static public function getCart() {
		if(!class_exists('Quick2cartModelcart'))
		{
			$path = JPATH_SITE.'/components/com_quick2cart/models/cart.php';
			JLoader::register('Quick2cartModelcart', $path );
			JLoader::load('Quick2cartModelcart');
		}
		$Quick2cartModelcart =  new Quick2cartModelcart();
		$cart_id = $Quick2cartModelcart->getCartId();
		$cart = $Quick2cartModelcart->getCartitems(false);
		return $cart;
	}	
	
	/**
	 * Query the cart model to retrieve the store id of a product
	 * (used to enforce "single store" carts in multivendor mode)
	 *
	 * @param unknown $product_id
	 */
	static public function getProductStoreId($product_id) {
		if(!class_exists('Quick2cartModelcart'))
		{
			$path = JPATH_SITE.'/components/com_quick2cart/models/cart.php';
			JLoader::register('Quick2cartModelcart', $path );
			JLoader::load('Quick2cartModelcart');
		}
		$Quick2cartModelcart =  new Quick2cartModelcart();
		$product = $Quick2cartModelcart->getItemRec($product_id);
		if ($product) {
			return $product->store_id;
		} else {
			JError::raiseWarning(500, "Unable to retrieve the store of item $product_id");
			return -1;
		}
	}

	/*
	 *    HTML Part =====================================================================
	 */
	
	/**
	 * Parse the selector, find in body, add the banner.
	 * @param unknown $selector
	 * @param unknown $banner
	 * @param unknown $body
	 */
	static public function addBanner($selector, $banner, $body) {
		$limit = 1;
		$count = 0;
		$bannerStyle = ' style="background-image:url('.$banner.');background-size:cover;background-repeat:no-repeat;" ';
		if (strpos($selector,'.')===0) {
			$elementClass = str_replace('.',' ', trim($selector, '. '));
			$body = preg_replace(
					'@(class=\s*["\']'.$elementClass.'[^"\']*["\'])@',
					'\1'.$bannerStyle,
					$body,
					$limit,
					$count);
		} else if (strpos($selector,'#')===0) {
			$elementId = ltrim($selector, '#');
			$body = preg_replace(
					'@(id=\s*["\']'.$elementId.'[^"\']*["\'])@',
					'\1'.$bannerStyle,
					$body,
					$limit,
					$count);
		} else {
			$elementTag = $selector;
			$body = preg_replace(
					'@(<'.$elementTag.'@',
					'\1'.$bannerStyle,
					$body,
					$limit,
					$count);
		}
		if ($count==0) {
			$body = preg_replace(
					'@(<body[^>]*)@',
					'\1<div class="qtcBanner"><img src="'.$banner.'" /></div>',
					$body);
	
		}
		return $body;
	}
	
	static public function getTermsAndPrivacyLinks($store_id, $showTerms=true,$showPrivacy=true,$spanWidth=6,$params) {

		if (!$showTerms && !$showPrivacy) {
			return "";
		}
		if (empty($store_id)) {
			if ($params->get('perShopTerms') || $params->get('perShopPrivacy')) {
				/*
					* we only need a store Id here if we use custom store id!
				* invoked without a store_id only during checkout.
				* getStoreId is pretty slow so we only want to call it if
				* it's absolutely necessary!
				*/
				$store_ids = self::getStoreId();
				if (count($store_ids)>0)
					$store_id = $store_ids[0];
			}
		}
	
		$termsUrl = 'index.php?action=qtc_popup_terms&store_id='.$store_id;
		$privacyUrl = 'index.php?action=qtc_popup_privacy&store_id='.$store_id;
	
		$html[] = self::loadColorboxInit();
		
		
		if ($spanWidth>0) {
			$html[] ="<div class='span$spanWidth'>";
		}
	
		if ($showTerms) {
			$html[] = sprintf("<a class='storepopup storeterms' href='%s'>Terms and conditions</a>&nbsp;&nbsp;", $termsUrl);
		}
		if ($spanWidth==0) {
			$html[] = '<br/>';
		}
		if ($showPrivacy) {
			$html[] = sprintf("<a class='storepopup storeprivacy' href='%s'>Privacy Policy</a>", $privacyUrl);
		}
	
		if ($spanWidth>0) {
			$html[] ="</div>";
		}
	
		return join("",$html);
	}
	
	
	static public function loadColorbox() {
		if (!self::$colorboxAdded) {
			error_log('loading colorbox');
			$document = JFactory::getDocument();
			$document->addScript('/plugins/system/qtc_fj_extra_fields/colorbox/js/jquery.colorbox-min.js');
			$document->addStyleSheet('/plugins/system/qtc_fj_extra_fields/colorbox/css/colorbox.css');
			self::$colorboxAdded = true;
		}
	}
	
	static public function loadColorboxInit() {
		$result = '';
		if (!self::$colorboxLoaded) {
			$result ="
					<script>
						jQuery(function() {
							   jQuery('.storeterms, .storeprivacy').colorbox({'height':'90%','width':'90%'});
						});
					</script>";
			self::$colorboxLoaded = true;
		}
		return $result;
	}
	
	/*
	 *     HTML INPUT PART =============================================================
	 */
	
	function getStoreCreateInputs($store_id,$params){
		
		//error_log('qtcOnBeforeCreateStore');
		$extra = self::getExtra($store_id);
	
		$html[] = "<h3>Extra fields</h3>";
		$html[] = self::buildExtraFieldHTML('fj_tax', 				'Tax (VAT)', 		$extra->tax,			'text');
		$html[] = self::buildExtraFieldHTML('fj_shipping', 		'Shipping', 		$extra->shipping,		'text');
		$html[] = self::buildExtraFieldHTML('fj_shipping_limit', 	'Shipping Limit', 	$extra->shippingLimit,	'text');
	
		
		if ($params->get('perShopTerms')==1) {
			$html[] = self::buildExtraFieldHTML('fj_per_shop_terms',
					'Terms and conditions', $extra->terms,	'textarea');
		}
		if ($params->get('perShopPrivacy')==1) {
			$html[] = self::buildExtraFieldHTML('fj_per_shop_privacy',
					'Privacy statement',	$extra->privacy,'textarea');
		}
		if ($params->get('perShopBanner')==1) {
			$html[] = self::buildExtraFieldHTML('fj_per_shop_banner',
					'Store Banner',	$extra->banner,'image');
				
		}
		return join('',$html);
	}
	
	/**
	 * A helper function to create the markup for the storecreate / store edit view fields.
	 * @param unknown $fieldName
	 * @param unknown $fieldDescription
	 * @param unknown $fieldValue
	 * @param string $fieldType
	 */
	static public function buildExtraFieldHTML($fieldName, $fieldDescription, $fieldValue, $fieldType='text') {
		$html[] = sprintf('<div class="control-group">
			<label class="control-label" for="%s">
			%s
			</label>
			<div class="controls">
			',$fieldName, $fieldDescription);
		switch($fieldType) {
			case 'text':
				$html[] = sprintf('<input name="%1$s" id="%1$s" type="text" value="%2$s"/>',
				$fieldName, $fieldValue);
				break;
			case 'textarea':
				$html[] = self::getEditor($fieldName, $fieldValue);
				break;
			case 'image':
				$html[] = self::getImageUpload($fieldName, $fieldValue);
				break;
			case 'raw':
				$html[] = $fieldValue;
				break;
		}
		$html[] = '</div></div>';
		return join ('',$html);
	}
	
	static private function getEditor($fieldName, $fieldValue) {
		// please see http://docs.joomla.org/JFactory/getEditor
		$editor = JFactory::getEditor();
		return $editor->display($fieldName, $fieldValue, "400", "100", "80", "10", false, $fieldName, null, null, array('mode' => 'advanced'));
	}


	static private function getImageUpload($fieldName, $fieldValue) {
		return sprintf('<input type="file" accept="image/*"
				placeholder="COM_Q2C_IMAGE_MSG"
				id="%1$s" name="%1$s" aria-invalid="false"/><br>
				<img src="%2$s" style="max-width:100%" />
	
				', $fieldName,$fieldValue);
	}
	
	static public   function getUploadedFile($store_id, $fieldName, $defaultValue, $params) {
		error_log('getUploadedFile');
		//error_log(var_export($post,true));
		// Import dependencies
		jimport('joomla.filesystem.file');
	
		// Get input
		$input = JFactory::getApplication()->input;
	
		// Get uploaded file info
		$file_info = $input->files->get($fieldName, null);
	
	
		// 		error
		// 		name
		// 		size
		// 		tmp_name
		// 		type
	
		error_log(var_export($file_info,true));
		// the file_info will also contain errors if any
	
	
		if ($file_info) {
			if (empty($file_info['error'])) {
				$destinationRelativePath = '/images/'.$params->get('bannerUploadPath').'/'.$store_id.'/';
				$destination = JPATH_BASE.$destinationRelativePath;
				$name = strtolower($file_info['name']);
				mkdir($destination,0755,true);
				// 	Move uploaded file destination
				if (JFile::upload($file_info['tmp_name'], $destination . $name)) {
					return JUri::base(true).$destinationRelativePath.$name;
				}
			}
	
		}
		return $defaultValue;
	}	

	/**
	 * Parse the input of an editor field;
	 * $defaultValue is used so in case users play with the settings they don't lose any
	 * data that was already saved.
	 *
	 * @param unknown $post
	 * @param unknown $fieldName
	 * @param unknown $defaultValue
	 */
	static public  function getPostHTML($post, $fieldName, $defaultValue='') {
		// 		$input_options = JFilterInput::getInstance(
		// 				array(
		// 						'img','p','a','u','i','b','strong','span','div','ul','li','ol','h1','h2','h3','h4','h5',
		// 						'table','tr','td','th','tbody','theader','tfooter','br'
		// 				),
		// 				array(
		// 						'src','width','height','alt','style','href','rel','target','align','valign','border','cellpading',
		// 						'cellspacing','title','id','class'
		// 				)
		// 		);
		// 		$input = new JInput($post, array('filter' => $input_options));
		// 		return $input->get('fieldName');
		//@TODO Add proper html parsing here, javascript could be injected here.
		return $post->get($fieldName,$defaultValue,'RAW');
	}
	

}

