<?php
/*
	* @package Quick2Cart FJ Extra Fields System plugin
	* @copyright Copyright (C) 2014 FasterJoomla. All rights reserved.
	* @license GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
	* @link     http://www.fasterjoomla.com
*/

defined( '_JEXEC' ) or die( 'Restricted access' );
jimport('joomla.plugin.plugin' );
jimport('joomla.filesystem.file');
jimport('joomla.application.application' );

class plgSystemQtc_fj_extra_fields extends JPlugin
{
	/**
	 * 
	 * 
	 * The first useful Joomla! event, used to:
	 * 
	 * 1. Test for the request of the terms and conditions and send it.
	 * 		This will respond to the following queries:
	 * 		?action=qtc_popup_terms
	 * 		?action=qtc_popup_privacy
	 * 
	 * 2. load the colorbox script in case
	 * 		the terms and conditions or privacy are enabled.
	 * 		(the buttons will be added directly to the markup in the onAfterRender event)
	 * 
	 * @return boolean
	 */
	public function onAfterRoute() {
		if (JPATH_BASE == JPATH_ADMINISTRATOR) {
			// do administrators deserve fancy touch icons? do they care? are they using a phone?
		 	return false;
		}
		$input = JFactory::getApplication()->input;
		//error_log('yes it\'s a store ');
		
		if (strpos($input->get('action'),'qtc_popup_')===0) {
			$this->popupCallback();
		}
		
		if ($this->isAllowed()) {
			$this->loadColorbox();
		}
		if ($this->isAllowed('productpage','')) {
			$this->loadColorbox();
		}
		
		/*if () {
			if ($this->params->get('loadJoomlaBootstrap')) {
				JHTML::b
			}
		}*/
	}
	
	/**
	 * Very ugly. We need to insert the terms and conditions in the store view
	 * but keep it this way to prevent overloading the view, so it will be easier
	 * to port to other installations.
	 * Inject the store terms and conditions here.
	
	 */
	public function onAfterRender() {
		if (JPATH_BASE == JPATH_ADMINISTRATOR) {
			// do administrators deserve fancy touch icons? do they care? are they using a phone?
			return false;
		}
		error_log('onAfterRender');
		$input = JFactory::getApplication()->input;
		$showTermsOrPrivacy = $this->isAllowed('vendor','store');
	
		$showTerms = $this->params->get("showTermsOnShop",1)==1;
		$showPrivacy = $this->params->get("showPrivacyOnShop",1)==1;
		$showBanner = ($this->params->get("perShopBanner",1)==1) && $input->get('tmpl')!=='component';
	
		// 		if ($this->isAllowed('productview','')) {
		//no questo si fa dalla view!
		// 			$document = JFactory::getDocument();
		// 			<script>
		// 						jQuery(function() {
		// 							   jQuery('.storeterms, .storeprivacy').colorbox({'height':'90%','width':'90%'});
		// 						});
		// 					</script>"
			
		// 		}
	
		if (!$showTerms && !$showPrivacy && !$showBanner) {
			return;
		}
		
		$store_id = $input->get('store_id');
		error_log('GET Store '.$store_id . ' view '. $input->get('view').'; layout: '.$input->get('layout'));
	
		$body = JResponse::getBody();
	
		if (($showTerms || $showPrivacy)&& $showTermsOrPrivacy) {
			// store id is surely set!
			$find = "@</address>\s*</div>@";
			//error_log('finding');
			$replace = "</address>".
					$this->getTermsAndPrivacyLinks($store_id,$showTerms,$showPrivacy).
					"</div>";
				
			$body = preg_replace($find, $replace, $body);
		}
	
		if ($showBanner) {
			$selector = $this->params->get('bannerContainerSelector');
			$banner = $this->getBannerImage($store_id);
				
			$body = $this->addBanner($selector, $banner, $body);
		}
		JResponse::setBody($body);
	}	


	private function popupCallback() {
		$input = JFactory::getApplication()->input;
		$action = $input->get('action');
		$store_id = $input->get('store_id');
		// even if store_id is null, we'll still load the default terms and conditions.
		$extra = $this->getExtra($store_id);
		switch ($action) {
			case 'qtc_popup_terms':
				echo $extra->terms;
				break;
			case 'qtc_popup_privacy':
				echo $extra->privacy;
					break;
		}
		exit; // we're showing terms and conditions in a popup, we don't want the website!
	}
	

	
	/**
	 * I want to show banners of the last store id used, unless the GET store_id param
	 * contains a store id, in which case that has the precedence;
	 * I additionally will store in session the store_id everytime 
	 * a product is added to the cart, so it will be available quickly for 
	 * pages where the store_id is not clearly defined.
	 * @TODO Finally, I want to ensure the store_id is cleared on certain pages where a 
	 * cart is empty.  
	 * 
	 * @param unknown $store_id
	 * @return unknown
	 */
	function getStoreIdFromRequest($store_id) {
		if (!empty($store_id)) {
			return $store_id;
		}
		$session = JFactory::getSession();
		$store_id = $session->get('active_cart_store_id',0);
		return $store_id;
	}
	
	/**
	 * Parse the selector, find in body, add the banner.
	 * @param unknown $selector
	 * @param unknown $banner
	 * @param unknown $body
	 */
	private function addBanner($selector, $banner, $body) {
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::addBanner($selector, $banner, $body);
	}
	
	/**
	 * The store terms and conditions are also made available above or below the cart
	 * during the checkout process:
	 */
	function OnBeforeq2cCheckoutCartDisplay()
	{
		
		$termsPos = $this->params->get("showTermsOnCheckout",1)==2;
		$privacyPos = $this->params->get("showPrivacyOnCheckout",1)==2;
		if ($termsPos || $privacyPos ) {
			$this->loadColorbox();
			error_log('OnBeforeq2cCheckoutCartDisplay ');
			return $this->getTermsAndPrivacyLinks(0, $termsPos, $privacyPos);
		}
	}
	
	function OnAfterq2cCheckoutCartDisplay()
	{
		$termsPos = $this->params->get("showTermsOnCheckout",1)==1;
		$privacyPos = $this->params->get("showPrivacyOnCheckout",1)==1;
		if ($termsPos || $privacyPos ) {
			$this->loadColorbox();
			error_log('OnAfterq2cCheckoutCartDisplay ');
			return $this->getTermsAndPrivacyLinks(0, $termsPos, $privacyPos);
		}
	}
	
	private function getBannerImage($store_id) {
		$store_id = $this->getStoreIdFromRequest($store_id);
		$extra = $this->getExtra($store_id);
		return $extra->banner;
	}
	
	public function getTermsAndPrivacyLinks($store_id, $showTerms=true, $showPrivacy=true, $spanWidth=6) {
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::getTermsAndPrivacyLinks($store_id, $showTerms, $showPrivacy, $spanWidth, $this->params);
	}
	
	/**
	 * Generate the HTML markup that will display at the bottom
	 * of the Create Store / Edit store window.
	 * @param int $store_id
	 */
	function qtcOnBeforeCreateStore($store_id){ 
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::getStoreCreateInputs($store_id,$this->params);
		
	}
	

	/** 
	 * Parse the extra fields fj_tax, fj_shipping
	 * which are added in the event qtcOnBeforeCreateStore,
	 * then add the values to the "extra" field so they 
	 * will be automatically persisted to the database
	 * @param JRegistry $post
	 * @return JRegistry
	 */
	function qtcOnBeforeSaveStore($post) {
 		//error_log('qtcOnBeforeSaveStore');
 		$input = JFactory::getApplication()->input;
		$id = $post->get('id');
		$extra = $this->getExtra($id);  // load the current values from the db; 
		// all the next fields default to the default value.
		$extra->tax = $post->get('fj_tax', $extra->tax);
		$extra->shipping = $post->get('fj_shipping', $extra->shipping);
		$extra->shippingLimit = $post->get('fj_shipping_limit', $extra->shippingLimit);
		
		require_once(__DIR__.'/helper.php');
		
		
		$extra->terms = fjExtraFieldsHelper::getPostHTML($post, 'fj_per_shop_terms', $extra->terms);//$post->get('fj_per_shop_terms','','RAW');
		$extra->privacy = fjExtraFieldsHelper::getPostHTML($post, 'fj_per_shop_privacy', $extra->privacy);//$post->get('fj_per_shop_privacy','','RAW');
		$extra->banner = fjExtraFieldsHelper::getUploadedFile($id, 'fj_per_shop_banner', $extra->banner,$this->params);
		
		$post->set('fj_tax',null);
		$post->set('fj_shipping',null);
		$post->set('fj_shipping_limit',null);
		$post->set('fj_per_shop_terms',null);
		$post->set('fj_per_shop_privacy',null);
		$post->set('fj_per_shop_banner',null);
		
		$post->set('extra',json_encode($extra));
		return $post;
	}


	
	/**
	 * We only need the colorbox on the store view 
	 * (it's used to popup the terms and conditions and privacy)
	 * @return boolean
	 */
	private function isAllowed($view='vendor',$layout='store') {
		if (JPATH_BASE == JPATH_ADMINISTRATOR) {
			// do administrators deserve fancy touch icons? do they care? are they using a phone?
			return false;
		}
		$document	= JFactory::getDocument();
		if ( $document->getType() != 'html' ) {
			return false;
		}
		$input = JFactory::getApplication()->input;
		if ($input->get('option')!=='com_quick2cart') {
			return false;
		}
		if ($input->get('view')!==$view) {
			return false;
		}
		if ($input->get('layout')!==$layout && !empty($layout)) {
			return false;
		}
		return true;
	}
	


	
	function qtcOnAfterSaveStore($post,$storeid) {
		//error_log('qtcOnAfterSaveStore');
	}
	
	function qtcaddTabOnCheckoutPage($cartData) {
		return;
		//load language file
		//$lang = JFactory::getLanguage();
		//$lang->load('plg_qtc_sample_development',
		return array('tabName'=>"store_details" , // tab name
			'nextstepid'=>"3"	,			// show tab
			'tabPlace'=>'4',  // require in future
			'html'=>'html to return');
	}
	
	
	/**
	 * Return an array (false,error message) when it's not 
	 * allowed to add items from different stores to the cart.
	 * Open this plugins' configuration to specify if you want to allow
	 * multivendor cart or not.
	 * @param unknown $cartId
	 * @param unknown $item
	 * @return void|multitype:boolean string
	 */
	function OnBeforeq2cAdd2Cart($cartId,$item)
	{
		error_log("OnBeforeq2cAdd2Cart\n======id: $cartId ========\n");
		$params = $this->params;
		if ($params->get('allowMultiVendorCart')==1) {
			return;
		}
		
		// if we got this far, only items from one store may be added to the 
		// cart at one time.
		// hence $store_ids should have at most 1 item.
		
		$store_ids = $this->getStoreId();
		
		if (count($store_ids)>1) {
			//JError::raiseWarning(518,"You are not supposed to order products from different vendors in the same cart.  Please clear some items before proceeding.");
			error_log('OnBeforeq2cAdd2Cart MULTIPLE VENDORS IN THE SAME CART '.$cartId);
			return array(false,"You are not supposed to order products from different vendors in the same cart.  Please open the cart and remove some items before proceeding.");
		}
		
		$productId = $item['id'];
		if (!isset($productId)) {
			$productId = $item['item_id']; // this happens when I order using the "People who bought this also bought" module
		}
		if (!isset($productId)) {
		
			error_log('OnBeforeq2cAdd2Cart - itemid not set '. var_export($item,true));
			return;
		}
		$product_store_id = $this->getProductStoreId($productId);
		
		if (count($store_ids)==1) {
			// else, the cart already contains items from one store.
			// we need to ensure this item does not belong to a different store:
			$store_id = $store_ids[0];
		
			// some prevention won't hurt
			error_log('OnBeforeq2cAdd2Cart - Item: '. var_export($item,true));
			error_log('store '.$store_id);
				
			
			
			if ($store_id != $product_store_id) {
				//JError::raiseWarning(500, "You are not allowed to add items from multiple stores to the same cart. Please complete your order with the current vendor before adding this item.");
				return array(false,"You are not allowed to add items from multiple stores to the same cart. Please complete your order with the current vendor before adding this item.");
			} else {
				error_log('default returning - 4 , all items from the same store');
				// just fine, all items belong to the same store
				// just a precaution, we should have set this already:
				$this->setStoreId($product_store_id);
				return; //
			}
			// 		error_log(var_export($cart,true));
			// 		error_log(var_export($item,true));
		} else 
		if (count($store_ids)==0) {
			// current cart is empty, proceed:
			$this->setStoreId($product_store_id);
			return;
		}
	}
	
	function setStoreId($store_id) {
		$session = JFactory::getSession();
		$session->set('active_cart_store_id',$store_id);
	}
	
	/**
	 * Try to catch the refunded state of the order.
	 * @param unknown $order
	 */
	function Onq2cOrderUpdate($order)
	{
		/*
		 * ric: let's determine if the payment is to be refunded
		* Another error: $order->status is always C!!! we 
		* need to invoke $comquick2cartHelper->getStoreItemStatus()
		*
		*/
		$comquick2cartHelper = new comquick2cartHelper();
		$application = JFactory::getApplication();
		
		//find out the store id!!! (an order can belong to multiple stores!)
		$store_id =  $application->input->get('store_id');
		if (empty($store_id) && (isset($order->store_id))) {
			$store_id = $order->store_id;
		}
		if (!empty($store_id)) {
			$order->status=$comquick2cartHelper->getStoreItemStatus($order->id,$store_id);
		}
	
		//error_log(var_export($order,true));
		if($order->status == 'RF'){
			// error_log('refund user!');
			$orderExtra = json_decode($order->extra);
			//error_log(var_export($orderExtra,true));
			$paymentResponse = $orderExtra->payment_response;
			$provider = "unknown payment provider";
			if (isset($paymentResponse->txn_type)) {
				$txn_type = $paymentResponse->txn_type;
				$provider = $txn_type;
			} else if (isset($paymentResponse->ipnNotificationUrl)) {
				$ipnN = $paymentResponse->ipnNotificationUrl;
				if (preg_match("/processor=adaptive_paypal/",$ipnN)) {
					$provider= ('Adaptive Paypal');
				} else {
					$provider = preg_replace("/^.*processor=//",'',$ipnN);
				}
			}
			
			//error_log(var_export(,true));
			JError::raiseNotice(500,'Order status updated, please refund the user from the payment gateway '.$provider);
			//$application->redirect('/index.php');
			//return;
		}
	}
	
	/**
	 * Invoked during checkout by the plugins qtc_fj_tax & qtc_fj_shipping
	 */
	public function getExtraFields() {
		$store_ids = $this->getStoreId();
		// this will make getExtra default to an empty but appropriate structure
		$store_id = 0; 
		if (count($store_ids)>0) {
			$store_id = $store_ids[0];
		}
		return $this->getExtra($store_id);
	}
	

	private function loadColorbox() {
		require_once(__DIR__.'/helper.php');
		fjExtraFieldsHelper::loadColorbox();
	}

	/**
	 * Return the current cart's store ids,
	 * additionally warning if items in the cart belong to more than one store.
	 *
	 * Multiple store_ids may imply multiple shipping charges; it is the responsibility
	 * of the caller to use one or more as appropriate.
	 *
	 * An additional param $raiseWarningOnMultipleStores may be set to raise a
	 * suspiciously vague error message.
	 *
	 * @return array
	 */
	private function getStoreId($raiseWarningOnMultipleStores = false) {
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::getStoreId($raiseWarningOnMultipleStores);
	}
	
	
	/**
	 * Retrieve the current extra record from the database;
	 * Used both when filling in the information on the store
	 * edit view and when saving the data, in order
	 * not to overwrite any values inserted by other plugins
	 *
	 * @param int $storeId
	 * @return stdClass
	 */
	public function getExtra($storeId) {
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::getExtra($storeId,$this->params);
	}
	
	/**
	 * Load an article directly from the db and return its content
	 * This is used to return the default article for terms and conditions and privacy.
	 * @param unknown $articleId
	 */
	private function loadContentArticle($articleId) {
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::loadContentArticle($articleId,$this->params);
	}
	
	/**
	 * Query the cart model to retrieve the store id of a product
	 * (this is used to enforce "single store" carts in multivendor mode)
	 *
	 * @param unknown $product_id
	 */
	private function getProductStoreId($product_id) {
		require_once(__DIR__.'/helper.php');
		return fjExtraFieldsHelper::getProductStoreId($product_id);
	}

	
}

